set echo on
set termout on
set feedback on
set linesize 120
set pagesize 500
set long 40000

rem thanks to: http://ora-00001.blogspot.com/2009/07/creating-rest-web-service-with-plsql.html
rem also see: http://www.oracle-base.com/articles/10g/dbms_epg_10gR2.php
rem and http://download.oracle.com/docs/cd/B28359_01/appdev.111/b28424/adfns_web.htm

cl scr

rem create and authorize dad as SCOTT

BEGIN
  DBMS_EPG.drop_dad 
  ( dad_name => 'hrmrestapi'
  );
end;  


BEGIN
  DBMS_EPG.create_dad 
  ( dad_name => 'hrmrestapi'
  , path => '/hrmapi/*'
  );
 dbms_epg.set_dad_attribute 
 ( dad_name => 'hrmrestapi'
 , attr_name => 'path-alias'
 , attr_value => 'rest');
 dbms_epg.set_dad_attribute 
 ( dad_name => 'hrmrestapi'
 , attr_name => 'path-alias-procedure'
 , attr_value => 'hrm_rest_api.handle_request');
  END;
/

begin
  DBMS_EPG.AUTHORIZE_DAD('hrmrestapi');
end;
/ 

rem http://xp-vm:2100/hrmapi/rest/department/10



pause

create or replace
package hrm_rest_api
is

procedure handle_request (p_path in varchar2);

end hrm_rest_api;
/

create public synonym hrm_rest_api for hrm_rest_api
/

grant execute on hrm_rest_api to public
/




create or replace
package body hrm_rest_api
is
/*
supports urls like:
http://xp-vm:2100/hrmapi/rest/departments/10/employees/7782/subordinates
and
http://xp-vm:2100/hrmapi/rest/employees/7782/subordinates
and
http://xp-vm:2100/hrmapi/rest/departments/10/employees
*/

TYPE t_array IS TABLE OF VARCHAR2(500) 
   INDEX BY BINARY_INTEGER; 

FUNCTION SPLIT (p_in_string VARCHAR2, p_delim VARCHAR2) RETURN t_array  ;
function sub_array(l_array_in t_array, l_from in number) return t_array;
procedure process_departments(l_path_elements in t_array) ;
procedure process_employees(p_path_elements in t_array, p_deptno in number default null) ;
procedure process_department(p_deptno in number, p_path_elements in t_array); 
procedure process_employee(p_path_elements in t_array, p_empno in number default null) ;

procedure debug(l_path_elements in t_array)
is
begin
for i in 1..l_path_elements.count loop
     htp.p(l_path_elements(i));
  end loop;
  
end;  
   
procedure handle_request (p_path in varchar2)
is
 l_path_elements t_array := split(p_path,'/');
begin
  if lower(l_path_elements(1))='departments'
  then
    process_departments(sub_array(l_path_elements, 2));
  elsif  lower(l_path_elements(1))='employees'
  then
    process_employees(sub_array(l_path_elements, 2), null);
  end if;
  exception
when others
then

 htp.p(sqlerrm);
end handle_request;

procedure process_departments(l_path_elements in t_array) 
is

begin
  if l_path_elements.count = 0 or l_path_elements(1)='all'
  then
    for l_rec in (select * from dept order by deptno) loop
      htp.p(l_rec.deptno || ';' || l_rec.dname || ';' || l_rec.loc || '');
    end loop;
  else 
     process_department(p_deptno => l_path_elements(1), p_path_elements => sub_array(l_path_elements,2));
  end if;
end;

procedure process_department(p_deptno in number, p_path_elements in t_array) 
is

begin
  if p_path_elements.count = 0 or p_path_elements(1)='all' 
  then
    for l_rec in (select * from dept where deptno = p_deptno) loop
      htp.p(l_rec.deptno || ';' || l_rec.dname || ';' || l_rec.loc || '');
    end loop;
  elsif p_path_elements(1)='employees'
  then
     process_employees(p_path_elements => sub_array(p_path_elements,2), p_deptno => p_deptno );
  end if;
end;

procedure process_employees(p_path_elements in t_array, p_deptno in number default null) 
is

begin
  if p_path_elements.count = 0 or p_path_elements(1)='all' 
  then
     for l_rec in (select * from emp where deptno = nvl(p_deptno, deptno)) loop
        htp.p(   l_rec.empno || ';' || l_rec.ename || ';' 
              || l_rec.job||';' || l_rec.mgr||';' 
              || to_char(l_rec.hiredate,'DD-MM-YYYY')||';' 
              || l_rec.sal);
      end loop;  
  else
    process_employee(p_path_elements => sub_array(p_path_elements,2), p_empno => p_path_elements(1) ) ;
  end if;
end;

procedure process_employee(p_path_elements in t_array, p_empno in number default null) 
is

begin

  if p_path_elements.count > 0 and p_path_elements(1)='subordinates' 
  then
     for l_rec in (select * from emp where mgr = p_empno) loop
        htp.p(   l_rec.empno || ';' || l_rec.ename || ';' 
              || l_rec.job||';' || l_rec.mgr||';' 
              || to_char(l_rec.hiredate,'DD-MM-YYYY')||';' 
              || l_rec.sal);
      end loop;  
      
  else
     for l_rec in (select * from emp where empno = p_empno) loop
        htp.p(   l_rec.empno || ';' || l_rec.ename || ';' 
              || l_rec.job||';' || l_rec.mgr||';' 
              || to_char(l_rec.hiredate,'DD-MM-YYYY')||';' 
              || l_rec.sal);
      end loop;  
  
  end if;
end;


function sub_array(l_array_in t_array, l_from in number) return t_array
is
  l_array t_array;
  l_index number:=1;
begin
 
  for i in 1..l_array_in.count loop
     if i>= l_from
     then
       l_array(l_index):= l_array_in(i);
       l_index:= l_index +1;
      end if;
  end loop;
 
 
  return l_array;
end;  


FUNCTION SPLIT (p_in_string VARCHAR2, p_delim VARCHAR2) RETURN t_array  
   IS 
    
      i       number :=0; 
      pos     number :=0; 
      lv_str  varchar2(50) := RTRIM( p_in_string, p_delim ) || p_delim;
       
   strings t_array; 
    
   BEGIN 
    
      -- determine first chuck of string   
      pos := instr(lv_str,p_delim,1,1); 
    
      -- while there are chunks left, loop  
      WHILE ( pos != 0) LOOP 
          
         -- increment counter  
         i := i + 1; 
          
         -- create array element for chuck of string  
         strings(i) := substr(lv_str,1,pos-1); 
          
         -- remove chunk from string  
         lv_str := substr(lv_str,pos+1,length(lv_str)); 
          
         -- determine next chunk  
         pos := instr(lv_str,p_delim,1,1); 
          
         -- no last chunk, add to array  
         IF pos = 0 THEN 
         
            strings(i+1) := substr(lv_str,1 ,length(lv_str)-1); 
          
         END IF; 
       
      END LOOP; 
    
      -- return array  
      RETURN strings; 
       
   END SPLIT; 
   
end hrm_rest_api;
/

