package nl.amis.plsql;


import java.math.BigDecimal;

import java.sql.CallableStatement;
import java.sql.Date;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Struct;
import java.sql.Timestamp;
import java.sql.Types;

import java.util.ArrayList;
import java.util.List;

import oracle.jdbc.OraclePreparedStatement;
import oracle.jdbc.OracleTypes;

import oracle.sql.STRUCT;


public class EmployeeCaller {


    public String getEmployeeName(Integer no) throws SQLException {
        String query = "{call get_employee_name(?,?)}";
        CallableStatement cs =
            ConnectionManager.getConnection().prepareCall(query);
        cs.setInt(1, no);
        cs.registerOutParameter(2, Types.CHAR);
        cs.execute();

        Object rs = cs.getObject(2);
        return rs.toString();
    }

    public Employee getEmployee(Integer no) throws SQLException {
        String query = "{call get_employee(?,?)}";
        CallableStatement cs =
            ConnectionManager.getConnection().prepareCall(query);
        cs.setInt(1, no);
        cs.registerOutParameter(2, Types.STRUCT, "EMPLOYEE_TYPE");
        cs.execute();

        Struct employee = (Struct)cs.getObject(2);
        Object[] attributes = employee.getAttributes();
        Employee e = new Employee();
        e.setId(((BigDecimal)attributes[0]).intValue());
        e.setName((String)attributes[1]);
        e.setJob((String)attributes[2]);
        e.setHiredate(new Date(((Timestamp)attributes[3]).getTime()));
        e.setDepartment((String)attributes[4]);
        return e;
    }


    public List<Employee> getEmployees(Integer departmentNumber,
                                       String job) throws SQLException {
        String query = "{call get_employees(?,?,?)}";
        CallableStatement cs =
            ConnectionManager.getConnection().prepareCall(query);
        if (departmentNumber==null) 
          cs.setNull(1, Types.INTEGER);
        else
          cs.setInt(1, departmentNumber);
        cs.setString(2, job);
        cs.registerOutParameter(3, Types.ARRAY, "EMPLOYEE_TABLE_TYPE");
        cs.execute();
        Object[] emps = (Object[])cs.getArray(3).getArray();
        // the objects returned in the ARRAY are in fact STRUCTs
        // these are created based on the EMPLOYEE_TYPE SQL TYPE definition

        List<Employee> employees = new ArrayList<Employee>();
        for (int i = 0; i < emps.length; i++) {
            Struct employee = (Struct)emps[i];
            Object[] attributes = employee.getAttributes();
            Employee e = new Employee();
            e.setId(((BigDecimal)attributes[0]).intValue());
            e.setName((String)attributes[1]);
            e.setJob((String)attributes[2]);
            e.setHiredate(new Date(((Timestamp)attributes[3]).getTime()));
            e.setDepartment((String)attributes[4]);
            employees.add(e);
        }
        return employees;
    }


  public void updateEmployee( Integer empno,
                              String name, Integer sal) throws SQLException {      
    OraclePreparedStatement pstmt = (OraclePreparedStatement)ConnectionManager.getConnection().prepareStatement(
           "update emp set ename = ? , sal = ? where empno = ? returning ename, sal into ?, ?");
    pstmt.setString(1, name);
    pstmt.setInt(2,sal);
    pstmt.setInt(3,empno);
    // register return parameters
    pstmt.registerReturnParameter(4, OracleTypes.VARCHAR);
    pstmt.registerReturnParameter(5, OracleTypes.INTEGER);
    // process the DML returning SQL statement
    pstmt.executeUpdate();
    System.out.println("Done update with name = "+name+" and salary = "+sal);
    ResultSet rset = pstmt.getReturnResultSet();
    Integer salary;
    String employeeName;
    if (rset.next())
    {
      salary = rset.getInt(2);
      System.out.println("Salary after update =  "+salary);
      employeeName = rset.getString(1);
      System.out.println("Name after update =  "+employeeName);
    }
    
  }


    public static void main(String[] args) throws SQLException {
//        System.out.println("Calling Get Employee for 7902. The result: " +
//                           new EmployeeCaller().getEmployee(7902).getName());
//        System.out.println("Calling Get Employee Name for 7902. The result: " +
//                           new EmployeeCaller().getEmployeeName(7902));
//      System.out.println("Calling Get Employees for department 10. The result: " +
//                         new EmployeeCaller().getEmployees(10, null).size()+" employees.");
//      System.out.println("Calling Get Employees for CLERKs. The result: " +
//                         new EmployeeCaller().getEmployees(null, "CLERK").size()+" employees.");
//      try {
//      System.out.println("Calling Get Employee for -1. The result: " +
//                         new EmployeeCaller().getEmployeeName(-1));
//      } catch (SQLException sqle) {
//        System.out.println(sqle.getErrorCode());
//        System.out.println(sqle.getSQLState());
//        System.out.println(sqle.getMessage());
//      }
      
      new EmployeeCaller().updateEmployee(7902,"Johnny", 4321 );
      
//      System.out.println("Calling Get Employee for -1. The result: " +
//                         new EmployeeCaller().getEmployeeName(-1));

    }
}
