package nl.amis.plsql;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;

import nl.amis.plsql.entities.Greeter;

public class HelloWorldJavaServiceFacade {
    private EntityManagerFactory emf =
        Persistence.createEntityManagerFactory("JavaAndPLSQL");

    public HelloWorldJavaServiceFacade() {
    }


    private EntityManager getEntityManager() {
        return emf.createEntityManager();
    }


    /*
     * Use Native Query and select FUNCTION from dual to retrieve function result
     */
    // JPA does not support IN and INOUT parameters for PL/SQL procedure calls
    //    JPA does not have any direct support for stored procedures. Some types of stored procedures can be executed in JPA through using native queries. Native queries in JPA allow any SQL that returns nothing, or returns a database result set to be executed.
    // The syntax to execute a stored procedure depends on the database. JPA does not support stored procedures that use OUTPUT or INOUT parameters.

    public String getGreeting(String name) {
        Query query =
            getEntityManager().createNativeQuery("select hello_world(P_NAME=>?) from dual");
        query.setParameter(1, name);
        String greeting = (String)query.getSingleResult();
        return greeting;
    }
    
  public String invokeGreeting(String name) {
      Query query =
          getEntityManager().createNativeQuery("hello_world_proc_no_out_param(P_NAME => ?)");
      query.setParameter(1, name);
      String greeting = (String)query.getSingleResult();
      return greeting;
  }

    public String getOtherGreeting(String name) {
        Query query = getEntityManager().createNamedQuery("RetrieveGreeting");
        query.setParameter("P_NAME", name);
        List l = query.getResultList();
        Object[] result = (Object[])query.getResultList().get(0);
        return (String)result[0];
    }


    /** <code>select o from Greeter o</code> */
    public List<Greeter> getGreeterFindAll() {
        return getEntityManager().createNamedQuery("Greeter.findAll").getResultList();
    }

    public static void main(String[] args) {
        final HelloWorldJavaServiceFacade helloWorldJavaServiceFacade =
            new HelloWorldJavaServiceFacade();
        //  TODO:  Call methods on helloWorldJavaServiceFacade here...
        List<Greeter> greeterList =
            helloWorldJavaServiceFacade.getGreeterFindAll();
        System.out.println(greeterList.get(0).getGreeting());
        System.out.println(helloWorldJavaServiceFacade.getGreeting("Johnny Doe"));
        System.out.println(helloWorldJavaServiceFacade.getOtherGreeting("Johnny D. Doe"));
    }

}
