create or replace view empdept_vw
as
select e.ename as name
,      d.dname as department
,      e.job
,      e.sal
,      e.hiredate
from   emp e
       left outer join
	   dept d
       on (e.deptno = d.deptno) 
/
	   

   
create or replace trigger handle_insert_empdept_trg
instead of insert on empdept_vw
for each row 
declare
  l_deptno emp.deptno%type := null;
begin
  if :new.department is not null
  then
    begin
      select deptno
      into   l_deptno
      from   dept
      where  dname = :new.department
      ;
	exception
	  when no_data_found
	  then
	    -- there is no department in DEPT 
		-- with name equals :new.department
		-- so we create one!
        select max(deptno) + 10
		into   l_deptno
        from   dept;
		insert into dept 
		(deptno, dname)
		values
        ( l_deptno , :new.department);
		/* the next statement is not supported; RETURNING INTO cannot be used together with a SUB Query in the insert statement 
	   insert into dept 
		(deptno, dname)
		values
        select max(deptno) + 10
		,      :new.department
		from   dept
		RETURNING deptno INTO l_deptno;			
	*/
	end;	
  end if;
  insert into emp
  (empno, ename, deptno, sal, hiredate, job)
  select max(empno) + 10
  ,      :new.name
  ,      l_deptno
  ,      :new.sal
  ,      :new.hiredate
  ,      :new.job
  from   emp
  ;
end;
/


insert into empdept_vw
( name, department, job, sal)
values
( 'ALEX', 'TRAINING' , 'TRAINER' , 7500)
/

select *
from   dept
/


insert into empdept_vw
( name, department, job, sal)
values
( 'LUCAS', 'TRAINING' , 'TRAINEE' , 2500)
/

select *
from   dept
/

select *
from   empdept_vw
where  department = 'TRAINING'
/

select *
from   emp
where  job like 'TR%'
/

update empdept_vw
set    department = 'SALES'
where  name = 'LUCAS'


delete empdept_vw
where  department = 'TRAINING'



create or replace trigger handle_upd_dept_empdept_trg
instead of update on empdept_vw
for each row 
declare
  l_deptno emp.deptno%type := null;
begin
  if updating('DEPARTMENT')
  then
    -- does an update of department mean:
    -- - update the name of the referenced department (for this employee AND all others)
    -- - a transfer of the employee to a department with the name specified
    if :new.department is not null
    then
      begin
        select deptno
        into   l_deptno
        from   dept
        where  dname = :new.department
        ;
	    exception
	      when no_data_found
	      then
	        -- there is no department in DEPT 
		      -- with name equals :new.department
		      -- so we create one!
          select max(deptno) + 10
		      into   l_deptno
          from   dept;
      		insert into dept 
		      (deptno, dname)
		      values
          ( l_deptno , :new.department);
      end;	
    end if;
    update emp
    set    deptno = l_deptno
    where  ename = :new.name;
  end if;
end;


update empdept_vw
set    department = 'FACTORY'
where  name = 'LUCAS'

