package nl.amis.webcache.model;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import javax.sql.DataSource;

import oracle.jdbc.OracleConnection;
import oracle.jdbc.OracleStatement;
import oracle.jdbc.dcn.DatabaseChangeEvent;
import oracle.jdbc.dcn.DatabaseChangeListener;
import oracle.jdbc.dcn.DatabaseChangeRegistration;


/**
 * very useful information in:
 * http://theblasfrompas.blogspot.com/2010/02/using-database-change-notification-dcn.html
 */
public class DatabaseNotificationCoordinator {
    private DatabaseChangeRegistration dcr = null;
    private static DatabaseNotificationCoordinator instance = null;

    static
    {
      try
      {
        instance = new DatabaseNotificationCoordinator();
      }
      catch (Exception e)
      {
        throw new RuntimeException("Error creating instance of DatabaseNotificationCoordinator", e);
      }
    }
    
    private DatabaseNotificationCoordinator () throws SQLException
    { 
      OracleConnection conn = getConnection();
      if (dcr == null)
      {
        registerDCN(conn);
      }
    }
    
    public static DatabaseNotificationCoordinator getInstance()
    {
      return instance;
    }
    
  private void registerDCN (OracleConnection conn) throws SQLException
  {
    /*
     * register a listener for change notofication to be displayed to standard out
     * for testing purposes
     */  
    Properties props = new Properties();
    props.setProperty(OracleConnection.DCN_QUERY_CHANGE_NOTIFICATION,"true");
    props.put(OracleConnection.DCN_NOTIFY_ROWIDS, "true");
    props.put(OracleConnection.NTF_QOS_RELIABLE, "false");
    props.setProperty(OracleConnection.DCN_BEST_EFFORT, "true");
    
    dcr = conn.registerDatabaseChangeNotification(props);
    
    // Add the dummy DCNListener which is DCNListener.java class
    DCNListener listener = new DCNListener();
    dcr.addListener(listener);

    Statement stmt = conn.createStatement();
    // Associate the statement with the registration.
    ((OracleStatement)stmt).setDatabaseChangeRegistration(dcr);
    ResultSet rs = stmt.executeQuery("select sal from emp where job='CLERK'");
    while (rs.next())
    {
      // do nothing no , need to just need query to register the DEPT table
    }

    String[] tableNames = dcr.getTables();
    for(int i=0; i < tableNames.length; i++)
    {
      System.out.println(tableNames[i]+" successfully registered.");
    }
    
    // close resources
    stmt.close();
    rs.close();

  }
  
  public void closeDCN (OracleConnection conn) throws SQLException
  {
    conn.unregisterDatabaseChangeNotification(dcr);
    conn.close();
  }

  

  public OracleConnection getConnection() {

    String jndiName = "java:/app/jdbc/jdbc/ScottXPVMDS";
    Context ctx=null;
      try {
          ctx = new InitialContext();
      } catch (NamingException e) {
      }
    DataSource ds=null;
      try {
          ds = (DataSource)ctx.lookup( jndiName);
      } catch (NamingException e) {
      }
      OracleConnection conn=null;
      try {
          conn = (OracleConnection)ds.getConnection();
      } catch (SQLException e) {
      }
      return conn;
  }

}
