create type income_type
as object
( deptno number(4)
, job varchar2(20)
, salary_sum number(10,2)
)
/

create type income_tbl_type
is table of income_type
/



create or replace package smart_cache
as

function get_income
return income_tbl_type
;

procedure invalidate_incomes_cache;

end smart_cache;
/

create or replace view incomes
( deptno
, job
, salary_sum
)
as 
select income.deptno
,      income.job
,      income.salary_sum
from   table( smart_cache.get_income ) income
/

create or replace package body smart_cache
as

g_incomes_cache income_tbl_type;
g_incomes_cache_valid varchar2(1):= 'N';

procedure invalidate_incomes_cache
is
begin
  g_incomes_cache_valid := 'N';
end;


function get_income
return income_tbl_type
is
begin
  if g_incomes_cache_valid = 'N'
  then  
    dbms_lock.sleep(2);
    select income_type( deptno, job, sum(sal))
    bulk   collect into g_incomes_cache
    from   emp
    group
    by     deptno
    ,      job
    ;
    g_incomes_cache_valid := 'Y';
  end if;
  return g_incomes_cache;
end;

end smart_cache;
/


set timing on

select * 
from   incomes
/

pause


select * 
from   incomes
/

pause

exec smart_cache.invalidate_incomes_cache;

pause

select * 
from   incomes
/

pause


select * 
from   incomes
/

pause

update emp
set    sal = sal *1.5
/

pause

select *
from   incomes
/

pause

select income_type( deptno, job, sum(sal))
from   emp
group
by     deptno
,      job
/

pause


exec smart_cache.invalidate_incomes_cache

pause

select * 
from   incomes
/

pause



create or replace trigger refresh_cache
 BEFORE DELETE OR INSERT OR UPDATE OF SAL, DEPTNO, JOB
 ON EMP
begin
   smart_cache.invalidate_incomes_cache;
end;
/

pause


exec smart_cache.invalidate_incomes_cache

pause

select * 
from   incomes
/

pause


update emp
set    sal = sal * 1.1
/
pause

select * 
from   incomes
/

pause

select * 
from   incomes
/

pause
