drop table t purge
/
drop function test_result_cache
/

cl scr

set echo off
prompt
prompt Start with a "regular" function
prompt
set echo on
create or replace
function test_result_cache (p_in in number)
   return number
is
begin
   dbms_lock.sleep (5);
   return p_in;
end;
/
pause

set timing on
set echo off
prompt
prompt ... and call this function a few times. 
prompt
set echo on

declare
   l_num number;
begin
   l_num := test_result_cache (1);
   l_num := test_result_cache (1);
end;
/

pause
set echo off
prompt
prompt Add the Oracle 11g RESULT_CACHE 
prompt

set echo on

create or replace
function test_result_cache (p_in in number)
   return number
   result_cache
is
begin
   dbms_lock.sleep (5);
   return p_in;
end;
/

pause
set echo off
prompt
prompt ... and watch the change in execution time 
prompt

set echo on

set timing on

declare
   l_num number;
begin
   l_num := test_result_cache (1);
   l_num := test_result_cache (1);
end;
/
pause
set echo off
prompt
prompt Call with the same input parameters 
prompt
set echo on

declare
   l_num number;
begin
   l_num := test_result_cache (1);
   l_num := test_result_cache (1);
end;
/
pause
set echo off
prompt
prompt Change the input parameters 
prompt
set echo on

declare
   l_num number;
begin
   l_num := test_result_cache (2);
   l_num := test_result_cache (2);
end;
/
pause
set echo off
prompt
prompt Result Cache Based on a Table 
prompt
set echo on

create table t
as
select rownum rn
  from all_objects
 where rownum <= 5
/
set echo off
prompt
prompt Just the RESULT_CACHE 
prompt
set echo on

create or replace
function test_result_cache (p_in in number)
   return number
   result_cache
is
   retval number;
begin
   select rn
     into retval
     from t
    where rn = p_in
   ;
   dbms_lock.sleep (1);
   return retval;
end;
/

pause

set echo off
prompt
prompt Call the function a few times 
prompt Watch the differences in execution times
prompt

set echo on
set timing on

declare
   l_num number;
begin
   for idx in 1..5
   loop
      l_num := test_result_cache (idx);
   end loop;
end;
/
pause
declare
   l_num number;
begin
   for idx in 1..5
   loop
      l_num := test_result_cache (idx);
   end loop;
end;
/
pause
set echo off
prompt 
prompt Change the content of the underlying table
prompt 
set echo on
update t
   set rn = -rn
/
commit;

pause

declare
   l_num number;
begin
   for idx in 1..5
   loop
      l_num := test_result_cache (idx);
   end loop;
end;
/
pause
set echo off
prompt
prompt RESULT_CACHE RELIES_ON Base table
prompt 
set echo on
create or replace
function test_result_cache (p_in in number)
   return number
   result_cache relies_on (t)
is
   retval number;
begin
   begin
      select rn
        into retval
       from t
       where rn = p_in
      ;
   exception
      when no_data_found
      then
         null;
   end;
   dbms_lock.sleep (1);
   return retval;
end;
/

pause

set timing on

declare
   l_num number;
begin
   for idx in 1..5
   loop
      l_num := test_result_cache (idx);
      dbms_output.put_line ('['||l_num||']');
   end loop;
end;
/
pause
declare
   l_num number;
begin
   for idx in 1..5
   loop
      l_num := test_result_cache (idx);
      dbms_output.put_line ('['||l_num||']');
   end loop;
end;
/
pause
set echo off
prompt
prompt Change the content of the underlying table
prompt
set echo on

update t
   set rn = -rn
;
commit;

pause
declare
   l_num number;
begin
   for idx in 1..5
   loop
      l_num := test_result_cache (idx);
      dbms_output.put_line ('['||l_num||']');
   end loop;
end;
/
pause
declare
   l_num number;
begin
   for idx in 1..5
   loop
      l_num := test_result_cache (idx);
      dbms_output.put_line ('['||l_num||']');
   end loop;
end;
/
pause
set echo off
prompt 
prompt Smart enough to get from the Cache what it needs
prompt   and go to the function when necessary
prompt
set echo on
declare
   l_num number;
begin
   for idx in 1..10
   loop
      l_num := test_result_cache (idx);
      dbms_output.put_line ('['||l_num||']');
   end loop;
end;
/
