rem webservice result cache

rem application context as cache
rem This Service Result Cache can be used on 9i through 11g (and not only 11g)
rem The 11g Result Cache cannot expire results, nor can it be refreshed at record level (it is cache level only)
rem This solution is limited by the maximum size of entries in a Global Application Context (either 4k or 32k); it should be extended probably by a result cache table that provides the overflow for the application context
rem (the application context could contain a reference to a record in a table that is created as  create table service_result_cache( id number(10) not null primary key, result clob)

create or replace package cache_mgr
as

procedure put_in_cache
( p_key    in varchar2
, p_value  in varchar2
);

function get_service_result_from_cache
( p_service        in varchar2
, p_parameter_hash in varchar2
) return varchar2
;

procedure put_service_result_in_cache
( p_service        in varchar2
, p_parameter_hash in varchar2
, p_expiry         in date
, p_result         in varchar2
)
;


end;
/


create or replace context global_cache using cache_mgr ACCESSED GLOBALLY
/



create or replace
package body cache_mgr
as

procedure put_in_cache
( p_key    in varchar2
, p_value  in varchar2
) is
begin
  DBMS_SESSION.SET_CONTEXT( 'global_cache',p_key ,p_value);
end put_in_cache;

function get_service_result_from_cache
( p_service        in varchar2
, p_parameter_hash in varchar2
) return varchar2
is
  l_key    varchar2(1000);
  l_value  varchar2(4000);
  l_expiry varchar2(20);
begin
  l_key:= p_service||'*;;*'||p_parameter_hash;
  l_value:= sys_context( 'global_cache', l_key);
  -- service results are postfixed with an expiry timestamp: format DD-MM-YY HH24:MI
  l_expiry := substr(l_value, instr(l_value,'*;;*') + 4);
  if to_date(l_expiry, 'DD-MM-YY HH24:MI') < systimestamp
  then
    -- remove value from cache as it is expired
    put_in_cache(l_key, null);  
    -- and return null    
    return null; 
  else
    return substr(l_value, 1, instr(l_value,'*;;*')-1);
  end if;
exception
when others
then
  -- check for ORA-2003 - invalid app context parameter (this means no entry was found for this key)
  return null;    
end get_service_result_from_cache;

procedure put_service_result_in_cache
( p_service        in varchar2
, p_parameter_hash in varchar2
, p_expiry         in date
, p_result         in varchar2
)is
  l_key    varchar2(1000);
  l_value  varchar2(4000);
  l_expiry varchar2(20):= to_char(p_expiry, 'DD-MM-YY HH24:MI');
begin
  l_key:= p_service||'*;;*'||p_parameter_hash;
  l_value:= p_result||'*;;*'||l_expiry;
  put_in_cache( l_key, l_value);
end put_service_result_in_cache;


end;
/



create or replace
function getsmartoilprice
return number
is
  l_oil_price number;
begin
  l_oil_price := to_number(
                       cache_mgr.get_service_result_from_cache
                       ( p_service => 'PURCHASING_PRICE'
                       , p_parameter_hash => 'OIL'
                       ));
  if l_oil_price is null
  then           
    -- call the (function that calls) the real WebService that provides the answer 
    l_oil_price := getOilPrice;
    cache_mgr.put_service_result_in_cache
    ( p_service        => 'PURCHASING_PRICE'
    , p_parameter_hash => 'OIL'
    , p_expiry         => sysdate + 1/24 -- one hour valid
    , p_result         => l_oil_price
    );
  end if;
  return l_oil_price;
end;
/


select getsmartoilprice
from   dual
/


  