create or replace 
function hello_world
( p_name in varchar2
) return varchar2
is
begin
  return 'Hello '||p_name;
end hello_world;


create or replace
view greeter
as
select hello_world( 'John Doe') greeting
from   dual


create or replace 
procedure hello_world_proc
( p_name in varchar2
, p_greeting in out varchar2
) 
is
begin
  p_greeting:= hello_world(p_name);
end hello_world_proc;



create or replace 
function hello_john
return varchar2
is
  l_greeting varchar2(100);
begin
  hello_world_proc('John Doe', l_greeting);
  return l_greeting;
end hello_world;


create or replace 
procedure hello_world_proc
( p_name in varchar2
, p_greeting in out varchar2
) 
is
begin
  p_greeting:= hello_world(p_name);
end hello_world_proc;

create or replace 
procedure hello_world_proc_no_out_param
( p_name in varchar2
) 
is
  l_greeting varchar2(100);
begin
  l_greeting:= hello_world(p_name);
end hello_world_proc_no_out_param;


create or replace 
procedure get_employee_name
( p_no   in number
, p_name out varchar2
) is
begin
  select ename
  into   p_name
  from   emp
  where  empno = p_no
  ;
end get_employee_name;



create or replace 
procedure get_employee_name
( p_no   in number
, p_name out varchar2
) is
begin
  select ename
  into   p_name
  from   emp
  where  empno = p_no
  ;
end get_employee_name;


create type employee_type
as object
( id         number(10)
, name       varchar2(100)
, job        varchar2(100)
, hiredate   date
, department varchar2(100)
)


create or replace
procedure get_employee
( p_no       in number
, p_employee out employee_type
) is
begin
  select employee_type( empno, ename, job, hiredate
                      , (select dname from dept where deptno = emp.deptno))
  into   p_employee
  from   emp
  where  empno = p_no
  ;
end get_employee;


create or replace
type employee_table_type as table of employee_type
/


create or replace
procedure get_employees
( p_department_number in number
, p_job               in varchar2
, p_employees         out employee_table_type
) is
  cursor c_emp ( b_deptno in number, b_job in varchar2)
    is
    select employee_type( empno, ename, job, hiredate
                         , (select dname from dept where deptno = emp.deptno)
                        ) employee
    from   emp
    where  deptno = nvl(b_deptno, deptno)
    and    job    = nvl(b_job, job)
  ;
  l_employees employee_table_type := employee_table_type();
begin
  for emp in c_emp( b_deptno => p_department_number
                  , b_job => p_job
                  ) loop
     l_employees.extend();
     l_employees(l_employees.last):= emp.employee;
  end loop;
  p_employees:= l_employees;
end get_employees;


create or replace
procedure get_employees
( p_department_number in number
, p_job               in varchar2
, p_employees         out employee_table_type
) is
begin
  select employee_type( empno, ename, job, hiredate
                      , (select dname from dept where deptno = emp.deptno)
  bulk collect into   p_employees
  from   emp
  where  deptno = nvl(p_department_number, deptno)
  and    job    = nvl(p_job, job)
  ;
end get_employees;


create or replace
procedure get_employees
( p_department_number in number
, p_job               in varchar2
, p_employees         out employee_table_type
) is
begin
  select cast( collect (employee_type( empno, ename, job, hiredate
                      , (select dname from dept where deptno = emp.deptno))
                  )
               as employee_table_type
              )
  into   p_employees
  from   emp
  where  deptno = nvl(p_department_number, deptno)
  and    job    = nvl(p_job, job)
  ;
end get_employees;





create type department_type
as object
( id       number(10)
, name     varchar2(100)
, location varchar2(100)
, staff    employee_table_type
)



create or replace 
function  get_employees_collection
( p_department_number in number
, p_job               in varchar2
) return employee_table_type
is
  l_employees employee_table_type;
begin
  get_employees
  ( p_department_number => p_department_number
  , p_job               => p_job
  , p_employees         => l_employees
  );
  return l_employees;
end get_employees_collection;


select *
from   table (get_employees_collection( 20,null))

  
  
declare
  l_name varchar2(100);
begin
  get_employee_name(-1, l_name);
end;  
  
  
create or replace 
procedure get_employee_name
( p_no   in number
, p_name out varchar2
) is
begin
  select ename
  into   p_name
  from   emp
  where  empno = p_no
  ;
exception
  when no_data_found
  then
    p_name:= 'UNKNOWN';
  when others
  then
    p_name:= '????';
end get_employee_name;


create or replace 
procedure get_employee_name
( p_no   in number
, p_name out varchar2
) is
begin
  if p_no < 1000
  then
    raise_application_error( -20001
                           , 'Unacceptable Input Parameter: '||p_no
                           ,false );
  end if;
  select ename
  into   p_name
  from   emp
  where  empno = p_no
  ;
exception
  when no_data_found
  then
    p_name:= 'UNKNOWN';
  when others
  then
    if SQLCODE = -20001
    then
      raise;
    end if;
    p_name:= '????';
end get_employee_name;


create or replace
function get_query_result
( p_query       in varchar2
, p_bind_param  in varchar2
, p_bind_param2 in varchar2
) return varchar2
is
  l_result varchar2(4000);
begin
  execute immediate p_query
  into l_result
  using p_bind_param, p_bind_param2
  ;
  return l_result;
end;


select get_query_result( 'select ename from emp where deptno = :1 and job= :2'
                       , 10
                       , 'MANAGER'
                       )
from   dual


create or replace 
package some_units
as
function get_query_result
( p_query       in varchar2
, p_bind_param  in varchar2
, p_bind_param2 in varchar2
) return varchar2;

function hello_world
( p_name in varchar2
) return varchar2;

procedure get_employee_name
( p_no   in number
, p_name out varchar2
);

end;
/


create or replace 
package body some_units
as

g_global varchar2(1000);
cursor emp(b_no in number)
is
  select ename
  from   emp
  where  empno = b_no
  ;

function get_query_result
( p_query       in varchar2
, p_bind_param  in varchar2
, p_bind_param2 in varchar2
) return varchar2
is
begin
  return get_query_result( p_query, p_bind_param, p_bind_param2);
end;  

function private_hello_world
( p_name in varchar2
) return varchar2
is
begin
  return 'Hello '||p_name;
end private_hello_world;

function hello_world
( p_name in varchar2
) return varchar2
is
begin
  return private_hello_world(p_name);
end hello_world;


procedure get_employee_name
( p_no   in number
, p_name out varchar2
) is
begin
  open emp(b_no => p_no);
  fetch emp into p_name;
  close emp;
exception
  when no_data_found
  then
    p_name:= 'UNKNOWN';
end get_employee_name;

begin
  g_global:= 'Welcome '
  ||to_char(sysdate, 'HH24:MI:SS');
end;
/


drop table
employee_dml_log
;

create table
employee_dml_log
( empno number(10)
, dml_timestamp timestamp default systimestamp
, performed_by varchar2(32) default user
, type_of_operation varchar2(1)
)
/

create or replace
trigger dml_emp_log
before insert or update or delete
on EMP
for each row
declare
  l_operation varchar2(1);
begin
  l_operation := case 
                 when inserting then 'I'
                 when updating then 'U'
                 when deleting then 'D'
                 end;
  insert into employee_dml_log
  (empno, type_of_operation)
  values
  (:new.empno, l_operation);
end;

begin
  insert into emp
  (empno, ename, job, deptno)
  values
  (2345, 'Jane D.', 'TRAINER', 20);
  rollback;
end;
/

select * 
from   emp
where  empno = 2345

select * 
from   employee_dml_log
where  empno = 2345


create or replace
trigger dml_emp_log
before insert or update or delete
on EMP
for each row
declare
  l_operation varchar2(1);
 PRAGMA AUTONOMOUS_TRANSACTION;
begin
  l_operation := case 
                 when inserting then 'I'
                 when updating then 'U'
                 when deleting then 'D'
                 end;
  insert into employee_dml_log
  (empno, type_of_operation)
  values
  (:new.empno, l_operation);
  commit;
end;

begin
  insert into emp
  (empno, ename, job, deptno)
  values
  (2345, 'Jane D.', 'TRAINER', 20);
  rollback;
end;
/

select * 
from   emp
where  empno = 2345

select * 
from   employee_dml_log
where  empno = 2345


CREATE OR REPLACE 
PACKAGE GREETER_PCK AS
 FUNCTION HELLO_WORLD(p_name IN VARCHAR2) RETURN VARCHAR2;
END GREETER_PCK;
/

CREATE OR REPLACE PACKAGE BODY GREETER_PCK AS
 FUNCTION HELLO_WORLD(p_name IN VARCHAR2) RETURN VARCHAR2
  AS LANGUAGE JAVA NAME 'nl.amis.greeter.GreetCoordinator.runThis(java.lang.String) return String';
END GREETER_PCK;
/

CREATE OR REPLACE AND COMPILE JAVA SOURCE NAMED "GreetCoordinator" AS
package nl.amis.greeter;

public class GreetCoordinator
{
  public static String runThis( java.lang.String name) {
    return "Hello - from Java "+name;
  }
}

select greeter_pck.hello_world('Nick Doe')
from   dual




create or replace 
procedure hrm_details
(t_cursor OUT sys_refcursor) 
is
begin
  open t_cursor for
  select empno id
  ,      ename name
  ,      job
  ,      case when mgr is not null then (select ename from emp m where m.empno = e.mgr) end boss
  ,      cursor (select ename, job from emp s where s.mgr = e.empno) staff
  from   emp e;
end;

create or replace 
function hrm_details_func
return sys_refcursor
is
  l_c sys_refcursor;
begin
  hrm_details(l_c);
  return l_c;
end hrm_details_func;


rem JOB

declare
  l_job_num number;
  l_job     varchar2(500):= 'begin insert into emp 
                                   (empno, ename, job, deptno, hiredate) 
                                   values
                                   ( (select max(empno)+1 from emp)
                                   , ''JACK''
                                   , ''DEVELOPER''
                                   , 10
                                   , sysdate
                                   );
                                  commit;
                              end;';    
begin
  dbms_job.submit
  ( l_job_num
  , l_job
  );
end;

select * from emp

commit


select * from emp

rem wait a few seconds for the background job to finish

select * from emp
