package nl.amis.plsql;

import java.sql.ResultSet;

import java.sql.SQLException;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;

import nl.amis.plsql.entities.Greeter;

public class EmployeeServiceFacade {
    private EntityManagerFactory emf =
        Persistence.createEntityManagerFactory("JavaAndPLSQL");


    private EntityManager getEntityManager() {
        return emf.createEntityManager();
    }


    /*
     * Use Native Query and select FUNCTION from dual to retrieve function result
     */
    // JPA does not support IN and INOUT parameters for PL/SQL procedure calls
    //    JPA does not have any direct support for stored procedures. Some types of stored procedures can be executed in JPA through using native queries. Native queries in JPA allow any SQL that returns nothing, or returns a database result set to be executed.
    // The syntax to execute a stored procedure depends on the database. JPA does not support stored procedures that use OUTPUT or INOUT parameters.

  public String invokeGreeting(String name) {
      Query query =
          getEntityManager().createNativeQuery("hello_world_proc_no_out_param(P_NAME => ?)");
      query.setParameter(1, name);
      String greeting = (String)query.getSingleResult();
      return greeting;
  }

    public String getOtherGreeting(String name) {
        Query query = getEntityManager().createNamedQuery("RetrieveGreeting");
        query.setParameter("P_NAME", name);
        List l = query.getResultList();
        Object[] result = (Object[])query.getResultList().get(0);
        return (String)result[0];
    }

  public String getEmployee(Integer id) {
      Query query = getEntityManager().createNamedQuery("RetrieveEmployee");
      query.setParameter("P_NO", id);
      List l = query.getResultList();
      Object[] result = (Object[])query.getResultList().get(0);
      return (String)result[0];
  }


  

  public void getHrm() throws SQLException {
      Query query = getEntityManager().createNamedQuery("RETRIEVE_HRM_VIA_CURSOR");
      for (int i=0; i< query.getResultList().size(); i++) {
        Object[] result = (Object[])query.getResultList().get(i);
        String name = (String)result[1];
        String job = (String)result[2];
        System.out.println("Employee "+name+" ("+job+")");
          ResultSet staff = (ResultSet)result[4];
          if (staff !=null) {
            while (staff.next()) {
              String staffname = staff.getString(0);
              System.out.println("**  Staff Name "+staffname);
            }              
          }
      }
  }


  public void retrieveHrmDetails() throws SQLException {
      Query query =
          getEntityManager().createNativeQuery
                             ("select hrm_details_func from dual");
      
      ResultSet results= (ResultSet)query.getSingleResult();
      while (results.next()) {
        String name = results.getString(1);
        String job = results.getString(2);
        System.out.println("Employee "+name+" ("+job+")");
        ResultSet staff = (ResultSet)results.getObject(4);
        while (staff.next()) {
          String staffname = staff.getString(0);
          System.out.println("**  Staff Name "+staffname);
        }                   
      }
  }


    public static void main(String[] args) throws SQLException {
        final EmployeeServiceFacade ef =
            new EmployeeServiceFacade();
        //  TODO:  Call methods on helloWorldJavaServiceFacade here...
       // System.out.println(ef.getEmployee(7902));
       // ef.retrieveHrmDetails();
        ef.getHrm();
    }

}

