SELECT 	XMLElement("Staff", 
  				  (SELECT XMLAgg( XMLElement("emp", 
  							      XMLAttributes(e.ename name), 
								  XMLElement( "Hiredate", hiredate),
                                  XMLForest( 
								            (SELECT XMLAgg( XMLElement("Subordinate", 
  												             XMLAttributes(s.ename "name")
														   )) 
  					                          FROM emp s 
  							 	              WHERE e.empno = s.mgr
                                             ) as "Subordinates"
                                           )
							    )) 
                   FROM   emp e 
                  )
				  ) AS "StaffMember" 
FROM  dual


select XMLType( cursor( select deptno
                        ,      dname
                        ,      cursor (select e.ename
                                       ,      e.job
                                       ,      e.sal 
                                       from   emp e
                                       where  e.deptno = d.deptno
                                      ) staff
                        from   dept d
                      )
              )
from  dual              



create or replace 
function get_hrm_xml
return XMLType
is
  l_xml xmltype;
  hrm   sys_refcursor;
begin
  open hrm for select XMLType( cursor( select deptno
                        ,      dname
                        ,      cursor (select e.ename
                                       ,      e.job
                                       ,      e.sal 
                                       from   emp e
                                       where  e.deptno = d.deptno
                                      ) staff
                        from   dept d
                      )
              )
  from  dual 
  ;
  fetch hrm into l_xml;
  close hrm;
  return l_xml;
end  get_hrm_xml;
/


select get_hrm_xml
from   dual
/



create or replace 
function get_nice_hrm_xml
return XMLType
is
  l_xml  xmltype := get_hrm_xml;
  l_xslt xmltype := XMLType(
            '<?xml version="1.0" ?>
             <xsl:stylesheet version="1.0" 
                             xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
               <xsl:template match="/ROWSET">
                 <Departments>
                   <xsl:for-each select="ROW">
                     <Department>
                       <Name><xsl:value-of select="DNAME"/></Name>
				       <Identifier><xsl:value-of select="DEPTNO"/></Identifier>
					   <Staff>
                         <xsl:for-each select="STAFF/STAFF_ROW">
                            <Employee>
                              <Name><xsl:value-of select="ENAME"/></Name>
				              <Job><xsl:value-of select="JOB"/></Job>
					        </Employee>
				         </xsl:for-each>
    				   </Staff>
					 </Department>
				   </xsl:for-each>
                 </Departments>				   
			</xsl:template>	 
      </xsl:stylesheet>
                           ');
begin
  return xmltype.transform(l_xml,l_xslt);
end  get_nice_hrm_xml;
/





create or replace 
package hrm_xml_api
as

function get_department_details
( p_deptno in number
) return XMLType
;

end hrm_xml_api;
/

create or replace 
package body hrm_xml_api
as

function get_department_details
( p_deptno in number
) return XMLType
is
cursor c_dept_xml (b_deptno in number)
is
SELECT 	XMLElement("Department", 
 					XMLAttributes(d.dname "name"), 
                    XMLElement("Staff", 
  					           (SELECT XMLAgg( XMLElement("emp", 
  										       XMLAttributes(e.ename name), 
                                               XMLForest( (SELECT XMLAgg( XMLElement("Subordinate", 
  												                          XMLAttributes(s.ename "name")
																	    )) 
  					                            		   FROM emp s 
  							 	                           WHERE e.empno = s.mgr
                                                          ) as "Subordinates"
                                                        )
											   ) 
  					           )
							   FROM   emp e 
   				               WHERE e.deptno =d.deptno)
                    ) AS "StaffMember" 
                  )           
FROM  dept d 
WHERE deptno = b_deptno; 
l_xml_type xmltype;
begin
  open c_dept_xml(b_deptno => p_deptno);
  fetch c_dept_xml into l_xml_type;
  close c_dept_xml;
  return l_xml_type;
end;

end;
/


select hrm_xml_api.get_department_details(10)
from dual
/

rem name of first employee in department 10:

select extractValue(hrm_xml_api.get_department_details(10),'//emp[1]/@NAME')
from dual
/

// count number of staff

select column_value as "Employee Count" 
from   xmltable( 'count(//emp)' 
                 passing hrm_xml_api.get_department_details(10)
               )
/
